<template>
  <Combobox
    :by="index"
    :model-value="modelValue"
    as="div"
    @update:model-value="$emit('update:modelValue', $event)"
  >
    <div class="relative">
      <ComboboxInput
        :id="id"
        :class="classes"
        :display-value="(option) => (option ? (option as Value)[label] : '')"
        class="w-full rounded-md border border-gray-900/10 bg-white py-1.5 pl-3 pr-10 text-gray-900 focus:ring-2 focus:ring-inset focus:ring-primary-500 sm:text-sm sm:leading-6"
        @change="$emit('search', $event.target.value)"
      />
      <ComboboxButton class="absolute inset-y-0 right-0 flex items-center rounded-r-md px-2 focus:outline-none">
        <ChevronDownRegularIcon
          aria-hidden="true"
          class="h-4 w-4 fill-gray-400"
        />
      </ComboboxButton>
      <ComboboxOptions
        v-if="options.length > 0"
        class="absolute z-10 mt-1 max-h-60 w-full overflow-auto rounded-md bg-white py-1 text-base shadow-lg ring-1 ring-black ring-opacity-5 focus:outline-none sm:text-sm"
        hold
      >
        <ComboboxOption
          v-for="(option, idx) in options"
          :key="idx"
          v-slot="{ active, selected }"
          :value="option"
          as="template"
        >
          <slot
            name="option"
            v-bind="{ option, active, selected }"
          >
            <li :class="['relative cursor-pointer select-none py-2 pl-3 pr-9', active ? 'bg-primary-500 text-white' : 'text-gray-900']">
              <span :class="['block truncate', selected && 'font-semibold']">
                {{ option[label] }}
              </span>
              <span
                v-if="selected"
                :class="['absolute inset-y-0 right-0 flex items-center pr-4', active ? 'text-white' : 'text-primary-500']"
              >
                <CheckRegularIcon
                  aria-hidden="true"
                  class="h-4 w-4 fill-current"
                />
              </span>
            </li>
          </slot>
        </ComboboxOption>
      </ComboboxOptions>
    </div>
  </Combobox>
</template>

<script lang="ts" setup>
import CheckRegularIcon from "@/svg/icons/check-regular.svg?component";
import ChevronDownRegularIcon from "@/svg/icons/chevron-down-regular.svg?component";
import { Combobox, ComboboxButton, ComboboxInput, ComboboxOption, ComboboxOptions } from "@headlessui/vue";
import type { ComputedRef } from "vue";

type Value = Record<any, any>;

defineProps<{
  index: keyof Value;
  label: keyof Value;
  options: Value[];
  modelValue: Value | null;
}>();

defineEmits<{
  search: [value: string];
  "update:modelValue": [value: Value | null];
}>();

const id = inject<string>("id");
const errors = inject<ComputedRef<boolean>>("errors");

const classes = computed(() => ({
  "text-red-600 border-red-500 placeholder:text-red-400 focus:ring-red-600": errors?.value,
}));
</script>
